@@ -7,7 +7,7 @@ from dateutil import parser
 from numba import jit
 from numba import typed
 import numpy as np
-
+from datetime import timedelta
 import os
 
 from napoleontoolbox.file_saver import dropbox_file_saver
@@ -413,11 +413,19 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
     balance_pkl_file_name = f'{filename_stub}_balance.pkl'
     transfer_pkl_file_name = f'{filename_stub}_transfer.pkl'
 
+    initial_balance_time = start_time
+    initial_balances = None
+    while (initial_balances is None or initial_balances.empty) and not initial_balance_time>end_time:
+        try:
+            initial_balances = binance_get_histo_future_balance(public_key, private_key, initial_balance_time)
+        except Exception as e:
+            print(e)
+        initial_balance_time = initial_balance_time + timedelta(days = 1)
+
+
+
     fut_transfers = binance_get_futures_transfer_histo(public_key, private_key, start_time, end_time)
 
-
-    initial_balances = binance_get_histo_future_balance(public_key, private_key, start_time)
-
     trades = binance_get(public_key, private_key, 'trade_history', start_time, end_time=end_time, symbol ='ETHUSDT').drop_duplicates()
     ohlc_eth_binance = binance_get(public_key, private_key, 'futures_klines', start_time, end_time= end_time, symbol ='ETHUSDT', aggreg ='1m')
     ohlc_btc_binance = binance_get(public_key, private_key, 'futures_klines', start_time, end_time= end_time, symbol ='BTCUSDT', aggreg ='1m')
@@ -425,7 +433,9 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
 
     ohlc_bnb_min_binance = binance_get(public_key, private_key, 'spot_klines', start_time, end_time=end_time, symbol ='BNBUSDT', aggreg ='1m')
 
+#    initial_balance_time =  min(pd.to_datetime(trades['time'], unit='ms'))
 
+ #   print(f'found balance {initial_balances} at {initial_balance_time}')
     # Get datacryptocompare
     cc = CryptoCompare('afb994efa0ca3fc08f6b4b4b1ce74d3cfdfbf962b4c2e6fe4a5694dd37cd68aa')
     startdate = start_time.strftime('%Y-%m-%dT%H:%M:%S')
@@ -597,26 +607,28 @@ def fetch_all_necessary_data(public_key = None, private_key= None, start_time= N
     else:
         wh['transfer_usd_bis'] = 0.
 
-    fut_transfers_bnb = fut_transfers[fut_transfers['asset'] == 'BNB'].copy()
-    fut_transfers_usd = fut_transfers[fut_transfers['asset'] == 'USDT'].copy()
 
-    fut_transfers_bnb = fut_transfers_bnb.set_index('date')
-    fut_transfers_usd = fut_transfers_usd.set_index('date')
+    if fut_transfers is not None:
+        fut_transfers_bnb = fut_transfers[fut_transfers['asset'] == 'BNB'].copy()
+        fut_transfers_bnb = fut_transfers_bnb.set_index('date')
+        fut_transfers_bnb['dt_floor_h'] = fut_transfers_bnb.index.map(lambda x: x.floor('H'))
+        fut_transfers_bnb = fut_transfers_bnb.groupby('dt_floor_h').sum()
 
-    fut_transfers_usd['dt_floor_h'] = fut_transfers_usd.index.map(lambda x: x.floor('H'))
-    fut_transfers_usd = fut_transfers_usd.groupby('dt_floor_h').sum()
 
-    fut_transfers_bnb['dt_floor_h'] = fut_transfers_bnb.index.map(lambda x: x.floor('H'))
-    fut_transfers_bnb = fut_transfers_bnb.groupby('dt_floor_h').sum()
+        fut_transfers_usd = fut_transfers[fut_transfers['asset'] == 'USDT'].copy()
+        fut_transfers_usd = fut_transfers_usd.set_index('date')
+        fut_transfers_usd['dt_floor_h'] = fut_transfers_usd.index.map(lambda x: x.floor('H'))
+        fut_transfers_usd = fut_transfers_usd.groupby('dt_floor_h').sum()
 
-    fut_transfers_usd = fut_transfers_usd[['amount']].copy()
-    fut_transfers_usd.columns = ['amount_futures_transfer_usd']
 
-    fut_transfers_bnb = fut_transfers_bnb[['amount']].copy()
-    fut_transfers_bnb.columns = ['amount_futures_transfer_bnb']
+        fut_transfers_usd = fut_transfers_usd[['amount']].copy()
+        fut_transfers_usd.columns = ['amount_futures_transfer_usd']
 
-    wh = pd.merge(wh, fut_transfers_usd, how='left', left_index=True, right_index=True)
-    wh = pd.merge(wh, fut_transfers_bnb, how='left', left_index=True, right_index=True)
+        fut_transfers_bnb = fut_transfers_bnb[['amount']].copy()
+        fut_transfers_bnb.columns = ['amount_futures_transfer_bnb']
+
+        wh = pd.merge(wh, fut_transfers_usd, how='left', left_index=True, right_index=True)
+        wh = pd.merge(wh, fut_transfers_bnb, how='left', left_index=True, right_index=True)
 
     initial_bnb_balance = 0
     initial_usdt_balance = 0
